@@ -39,7 +39,8 @@ typedef enum  {
 	TRACKER_DATA_ERROR_UNKNOWN_CLASS,
 	TRACKER_DATA_ERROR_UNKNOWN_PROPERTY,
 	TRACKER_DATA_ERROR_INVALID_TYPE,
-	TRACKER_DATA_ERROR_CONSTRAINT
+	TRACKER_DATA_ERROR_CONSTRAINT,
+	TRACKER_DATA_ERROR_NO_SPACE
 } TrackerDataError;
 
 typedef void (*TrackerStatementCallback) (const gchar *graph,
@@ -85,6 +86,7 @@ GPtrArray *
 tracker_data_update_sparql_blank           (const gchar               *update,
                                             GError                   **error);
 void     tracker_data_update_buffer_flush           (GError                   **error);
+void     tracker_data_update_buffer_might_flush     (GError                   **error);
 
 void     tracker_data_sync                          (void);
 
