@@ -102,6 +102,8 @@ struct TrackerMinerFSPrivate {
 
 	GQuark          quark_ignore_file;
 
+	GList          *config_directories;
+
 	GList          *directories;
 	DirectoryData  *current_directory;
 
@@ -604,9 +606,15 @@ fs_finalize (GObject *object)
 
 	g_free (priv->current_parent_urn);
 
-	if (priv->directories) {
-		g_list_foreach (priv->directories, (GFunc) directory_data_free, NULL);
-		g_list_free (priv->directories);
+	/* Not a leak, contents are actually owned
+	 * by priv->config_directories, being freed
+	 * right below.
+	 */
+	g_list_free (priv->directories);
+
+	if (priv->config_directories) {
+		g_list_foreach (priv->config_directories, (GFunc) directory_data_free, NULL);
+		g_list_free (priv->config_directories);
 	}
 
 	g_queue_foreach (priv->crawled_directories, (GFunc) crawled_directory_data_free, NULL);
@@ -919,6 +927,7 @@ sparql_update_cb (GObject      *object,
 
 	if (error) {
 		g_critical ("Could not execute sparql: %s", error->message);
+		priv->total_files_notified_error++;
 		g_error_free (error);
 	} else {
 		if (fs->private->been_crawled) {
@@ -1024,6 +1033,47 @@ item_query_exists (TrackerMinerFS  *miner,
 	return result;
 }
 
+static gboolean
+do_process_file (TrackerMinerFS *fs,
+		 ProcessData    *data)
+{
+	TrackerMinerFSPrivate *priv;
+	gboolean processing;
+
+	priv = fs->private;
+
+	g_signal_emit (fs, signals[PROCESS_FILE], 0,
+	               data->file, data->builder, data->cancellable,
+	               &processing);
+
+	if (!processing) {
+		gchar *uri;
+
+		uri = g_file_get_uri (data->file);
+
+		/* Re-fetch data, since it might have been
+		 * removed in broken implementations
+		 */
+		data = process_data_find (fs, data->file);
+
+		g_message ("%s refused to process '%s'", G_OBJECT_TYPE_NAME (fs), uri);
+
+		if (!data) {
+			g_critical ("%s has returned FALSE in ::process-file for '%s', "
+			            "but it seems that this file has been processed through "
+			            "tracker_miner_fs_file_notify(), this is an "
+			            "implementation error", G_OBJECT_TYPE_NAME (fs), uri);
+		} else {
+			priv->processing_pool = g_list_remove (priv->processing_pool, data);
+			process_data_free (data);
+		}
+
+		g_free (uri);
+	}
+
+	return processing;
+}
+
 static void
 item_add_or_update_cb (TrackerMinerFS *fs,
                        ProcessData    *data,
@@ -1034,17 +1084,43 @@ item_add_or_update_cb (TrackerMinerFS *fs,
 	uri = g_file_get_uri (data->file);
 
 	if (error) {
-		if (error->code == G_IO_ERROR_NOT_FOUND) {
-			g_message ("Could not process '%s': %s", uri, error->message);
+		ProcessData *first_item_data;
+		GList *last;
+
+		last = g_list_last (fs->private->processing_pool);
+		first_item_data = last->data;
+
+		/* Perhaps this is too specific to TrackerMinerFiles, if the extractor
+		 * is choking on some file, the miner will get a timeout for all files
+		 * being currently processed, but the one that is actually causing it
+		 * is the first one that was added to the processing pool, so we retry
+		 * the others.
+		 */
+		if (data != first_item_data &&
+		    (error->code == DBUS_GERROR_NO_REPLY ||
+		     error->code == DBUS_GERROR_TIMEOUT ||
+		     error->code == DBUS_GERROR_TIMED_OUT)) {
+			g_debug ("  Got DBus timeout error on '%s', but it could not be caused by it. Retrying file.", uri);
+
+			/* Reset the TrackerSparqlBuilder */
+			g_object_unref (data->builder);
+			data->builder = tracker_sparql_builder_new_update ();
+
+			do_process_file (fs, data);
 		} else {
-			g_critical ("Could not process '%s': %s", uri, error->message);
+			if (error->code == G_IO_ERROR_NOT_FOUND) {
+				g_message ("Could not process '%s': %s", uri, error->message);
+			} else {
+				g_critical ("Could not process '%s': %s", uri, error->message);
+			}
+
+			fs->private->total_files_notified_error++;
+			fs->private->processing_pool =
+				g_list_remove (fs->private->processing_pool, data);
+			process_data_free (data);
+
+			item_queue_handlers_set_up (fs);
 		}
-
-		fs->private->processing_pool =
-			g_list_remove (fs->private->processing_pool, data);
-		process_data_free (data);
-
-		item_queue_handlers_set_up (fs);
 	} else {
 		gchar *full_sparql;
 
@@ -1071,7 +1147,7 @@ item_add_or_update (TrackerMinerFS *fs,
 	TrackerMinerFSPrivate *priv;
 	TrackerSparqlBuilder *sparql;
 	GCancellable *cancellable;
-	gboolean processing, retval;
+	gboolean retval;
 	ProcessData *data;
 	GFile *parent;
 	gchar *urn;
@@ -1117,34 +1193,7 @@ item_add_or_update (TrackerMinerFS *fs,
 	data = process_data_new (file, urn, parent_urn, cancellable, sparql);
 	priv->processing_pool = g_list_prepend (priv->processing_pool, data);
 
-	g_signal_emit (fs, signals[PROCESS_FILE], 0,
-	               file, sparql, cancellable,
-	               &processing);
-
-	if (!processing) {
-		gchar *uri;
-
-		uri = g_file_get_uri (file);
-
-		/* Re-fetch data, since it might have been
-		 * removed in broken implementations
-		 */
-		data = process_data_find (fs, file);
-
-		g_message ("%s refused to process '%s'", G_OBJECT_TYPE_NAME (fs), uri);
-
-		if (!data) {
-			g_critical ("%s has returned FALSE in ::process-file for '%s', "
-			            "but it seems that this file has been processed through "
-			            "tracker_miner_fs_file_notify(), this is an "
-			            "implementation error", G_OBJECT_TYPE_NAME (fs), uri);
-		} else {
-			priv->processing_pool = g_list_remove (priv->processing_pool, data);
-			process_data_free (data);
-		}
-
-		g_free (uri);
-	} else {
+	if (do_process_file (fs, data)) {
 		guint length;
 
 		length = g_list_length (priv->processing_pool);
@@ -1159,6 +1208,7 @@ item_add_or_update (TrackerMinerFS *fs,
 	g_object_unref (file);
 	g_object_unref (cancellable);
 	g_object_unref (sparql);
+	g_free (urn);
 
 	return retval;
 }
@@ -1448,6 +1498,7 @@ item_move (TrackerMinerFS *fs,
 		/* Destination file has gone away, ignore dest file and remove source if any */
 		retval = item_remove (fs, source_file);
 
+		g_free (source_iri);
 		g_free (source_uri);
 		g_free (uri);
 
@@ -1557,9 +1608,10 @@ fill_in_queue (TrackerMinerFS       *fs,
 		file = node->data;
 		dir_data->n_items_processed++;
 
+		g_queue_push_tail (dir_data->nodes, node);
+
 		if (!g_object_get_qdata (G_OBJECT (file), fs->private->quark_ignore_file)) {
 			g_queue_push_tail (queue, g_object_ref (file));
-			g_queue_push_tail (dir_data->nodes, node);
 			return;
 		}
 	}
@@ -1648,7 +1700,14 @@ item_queue_get_next_file (TrackerMinerFS  *fs,
 			*source_file = NULL;
 			return QUEUE_WAIT;
 		} else {
-			fill_in_queue (fs, fs->private->items_created);
+			/* Iterate through all directory hierarchies until
+			 * one of these return something for the miner to do,
+			 * or no data is left to process.
+			 */
+			while (g_queue_is_empty (fs->private->items_created) &&
+			       !g_queue_is_empty (fs->private->crawled_directories)) {
+				fill_in_queue (fs, fs->private->items_created);
+			}
 		}
 	}
 
@@ -2241,7 +2300,6 @@ crawler_check_directory_cb (TrackerCrawler *crawler,
 {
 	TrackerMinerFS *fs = user_data;
 	gboolean should_check, should_change_index;
-	gboolean add_monitor = TRUE;
 
 	should_check = should_check_file (fs, file, TRUE);
 	should_change_index = should_change_index_for_file (fs, file);
@@ -2256,13 +2314,9 @@ crawler_check_directory_cb (TrackerCrawler *crawler,
 		                    GINT_TO_POINTER (TRUE));
 	}
 
-	g_signal_emit (fs, signals[MONITOR_DIRECTORY], 0, file, &add_monitor);
-
-	/* FIXME: Should we add here or when we process the queue in
-	 * the finished sig?
-	 */
-	if (add_monitor) {
-		tracker_monitor_add (fs->private->monitor, file);
+	if (!should_check) {
+		/* Remove monitors if any */
+		tracker_monitor_remove (fs->private->monitor, file);
 	}
 
 	/* We _HAVE_ to check ALL directories because mtime updates
@@ -2283,10 +2337,24 @@ crawler_check_directory_contents_cb (TrackerCrawler *crawler,
                                      gpointer        user_data)
 {
 	TrackerMinerFS *fs = user_data;
+	gboolean add_monitor = FALSE;
 	gboolean process;
 
 	g_signal_emit (fs, signals[CHECK_DIRECTORY_CONTENTS], 0, parent, children, &process);
 
+	if (process) {
+		g_signal_emit (fs, signals[MONITOR_DIRECTORY], 0, parent, &add_monitor);
+	}
+
+	/* FIXME: Should we add here or when we process the queue in
+	 * the finished sig?
+	 */
+	if (add_monitor) {
+		tracker_monitor_add (fs->private->monitor, parent);
+	} else {
+		tracker_monitor_remove (fs->private->monitor, parent);
+	}
+
 	return process;
 }
 
@@ -2348,6 +2416,8 @@ crawled_directory_data_free (CrawledDirectoryData *data)
 			 NULL);
 	g_node_destroy (data->tree);
 
+	g_queue_free (data->nodes);
+
 	g_slice_free (CrawledDirectoryData, data);
 }
 
@@ -2409,7 +2479,6 @@ crawler_finished_cb (TrackerCrawler *crawler,
 	           was_interrupted ? "Stopped" : "Finished",
 	           g_timer_elapsed (fs->private->timer, NULL));
 
-	directory_data_free (fs->private->current_directory);
 	fs->private->current_directory = NULL;
 
 	/* Proceed to next thing to process */
@@ -2466,7 +2535,6 @@ crawl_directories_cb (gpointer user_data)
 	}
 
 	/* Directory couldn't be processed */
-	directory_data_free (fs->private->current_directory);
 	fs->private->current_directory = NULL;
 
 	return TRUE;
@@ -2533,12 +2601,18 @@ tracker_miner_fs_directory_add (TrackerMinerFS *fs,
                                 GFile          *file,
                                 gboolean        recurse)
 {
+	DirectoryData *dir_data;
+
 	g_return_if_fail (TRACKER_IS_MINER_FS (fs));
 	g_return_if_fail (G_IS_FILE (file));
 
+	dir_data = directory_data_new (file, recurse);
+
+	fs->private->config_directories =
+		g_list_append (fs->private->config_directories, dir_data);
+
 	fs->private->directories =
-		g_list_append (fs->private->directories,
-		               directory_data_new (file, recurse));
+		g_list_append (fs->private->directories, dir_data);
 
 	crawl_directories_start (fs);
 }
@@ -2610,12 +2684,27 @@ tracker_miner_fs_directory_remove (TrackerMinerFS *fs,
 
 		if (g_file_equal (file, data->file) ||
 		    g_file_has_prefix (file, data->file)) {
+			fs->private->directories = g_list_delete_link (fs->private->directories, link);
+			return_val = TRUE;
+		}
+	}
+
+	dirs = fs->private->config_directories;
+
+	while (dirs) {
+		DirectoryData *data = dirs->data;
+		GList *link = dirs;
+
+		dirs = dirs->next;
+
+		if (g_file_equal (file, data->file)) {
 			directory_data_free (data);
 			fs->private->directories = g_list_delete_link (fs->private->directories, link);
 			return_val = TRUE;
 		}
 	}
 
+
 	/* Remove anything contained in the removed directory
 	 * from all relevant processing queues.
 	 */
@@ -2695,10 +2784,6 @@ tracker_miner_fs_file_notify (TrackerMinerFS *fs,
 
 	fs->private->total_files_notified++;
 
-	if (error) {
-		fs->private->total_files_notified_error++;
-	}
-
 	data = process_data_find (fs, file);
 
 	if (!data) {
@@ -2858,3 +2943,18 @@ tracker_miner_fs_get_parent_urn (TrackerMinerFS *fs,
 
 	return data->parent_urn;
 }
+
+void
+tracker_miner_fs_force_recheck (TrackerMinerFS *fs)
+{
+	GList *directories;
+
+	g_return_if_fail (TRACKER_IS_MINER_FS (fs));
+
+	g_message ("Forcing re-check on all index directories");
+
+	directories = g_list_copy (fs->private->config_directories);
+	fs->private->directories = g_list_concat (fs->private->directories, directories);
+
+	crawl_directories_start (fs);
+}
