@@ -187,12 +187,7 @@ crawler_check_file_cb (TrackerCrawler *crawler,
 
 static void
 crawler_finished_cb (TrackerCrawler *crawler,
-                     GQueue         *found,
                      gboolean        was_interrupted,
-                     guint           directories_found,
-                     guint           directories_ignored,
-                     guint           files_found,
-                     guint           files_ignored,
                      gpointer        user_data)
 {
 	g_main_loop_quit (user_data);
@@ -422,6 +417,13 @@ main (int argc, char **argv)
 		manager = tracker_miner_manager_new ();
 		miners = tracker_miner_manager_get_available (manager);
 
+		if (hard_reset || soft_reset) {
+			g_print ("%s\n", _("Waiting one second before starting miners…"));
+
+			/* Give a second's grace to avoid race conditions */
+			g_usleep (G_USEC_PER_SEC);
+		}
+
 		g_print ("%s\n", _("Starting miners…"));
 
 		/* Get the status of all miners, this will start all
