@@ -77,6 +77,8 @@ struct TrackerMinerFilesPrivate {
 
 	GQuark quark_mount_point_uuid;
 	GQuark quark_directory_config_root;
+
+	guint force_recheck_id;
 };
 
 enum {
@@ -106,6 +108,8 @@ static void        mount_pre_unmount_cb                 (GVolumeMonitor       *v
 static void        mount_point_added_cb                 (TrackerStorage       *storage,
                                                          const gchar          *uuid,
                                                          const gchar          *mount_point,
+                                                         gboolean              removable,
+                                                         gboolean              optical,
                                                          gpointer              user_data);
 static void        mount_point_removed_cb               (TrackerStorage       *storage,
                                                          const gchar          *uuid,
@@ -128,6 +132,9 @@ static void        index_recursive_directories_cb       (GObject              *g
 static void        index_single_directories_cb          (GObject              *gobject,
                                                          GParamSpec           *arg1,
                                                          gpointer              user_data);
+static void        ignore_directories_cb                (GObject              *gobject,
+							 GParamSpec           *arg1,
+							 gpointer              user_data);
 static DBusGProxy *extractor_create_proxy               (void);
 static gboolean    miner_files_check_file               (TrackerMinerFS       *fs,
                                                          GFile                *file);
@@ -302,6 +309,11 @@ miner_files_finalize (GObject *object)
 	                                      object);
 	g_object_unref (priv->volume_monitor);
 
+	if (priv->force_recheck_id) {
+		g_source_remove (priv->force_recheck_id);
+		priv->force_recheck_id = 0;
+	}
+
 	G_OBJECT_CLASS (tracker_miner_files_parent_class)->finalize (object);
 }
 
@@ -313,6 +325,8 @@ miner_files_constructed (GObject *object)
 	GSList *dirs;
 	GSList *mounts = NULL, *m;
 	gboolean index_removable_devices;
+	gboolean index_optical_discs;
+	TrackerStorageType type = 0;
 
 	G_OBJECT_CLASS (tracker_miner_files_parent_class)->constructed (object);
 
@@ -324,12 +338,18 @@ miner_files_constructed (GObject *object)
 		g_assert_not_reached ();
 	}
 
-	index_removable_devices = tracker_config_get_index_removable_devices (mf->private->config);
-
-	if (index_removable_devices) {
-		mounts = tracker_storage_get_removable_device_roots (mf->private->storage);
+	if (tracker_config_get_index_removable_devices (mf->private->config)) {
+		index_removable_devices = TRUE;
+		type |= TRACKER_STORAGE_REMOVABLE;
 	}
 
+	if (tracker_config_get_index_optical_discs (mf->private->config)) {
+		index_optical_discs = TRUE;
+		type |= TRACKER_STORAGE_OPTICAL;
+	}
+
+	mounts = tracker_storage_get_device_roots (mf->private->storage, type, TRUE);
+
 #if defined(HAVE_DEVKIT_POWER) || defined(HAVE_HAL)
 	check_battery_status (mf);
 #endif /* defined(HAVE_DEVKIT_POWER) || defined(HAVE_HAL) */
@@ -431,10 +451,15 @@ miner_files_constructed (GObject *object)
 		g_object_unref (file);
 	}
 
-	/* Add removable media */
-	g_message ("Setting up directories to iterate which are removable devices");
+	/* Add mounts */
+	g_message ("Setting up directories to iterate from devices/discs");
+
 	if (!index_removable_devices) {
-		g_message ("  Disabled in the config");
+		g_message ("  Removable devices are disabled in the config");
+	}
+
+	if (!index_optical_discs) {
+		g_message ("  Optical discs are disabled in the config");
 	}
 
 	for (m = mounts; m; m = m->next) {
@@ -456,6 +481,8 @@ miner_files_constructed (GObject *object)
 		g_object_unref (file);
 	}
 
+	/* Add optical media */
+
 	g_signal_connect (mf->private->config, "notify::low-disk-space-limit",
 	                  G_CALLBACK (low_disk_space_limit_cb),
 	                  mf);
@@ -465,6 +492,12 @@ miner_files_constructed (GObject *object)
 	g_signal_connect (mf->private->config, "notify::index-single-directories",
 	                  G_CALLBACK (index_single_directories_cb),
 	                  mf);
+	g_signal_connect (mf->private->config, "notify::ignored-directories",
+	                  G_CALLBACK (ignore_directories_cb),
+	                  mf);
+	g_signal_connect (mf->private->config, "notify::ignored-directories-with-content",
+	                  G_CALLBACK (ignore_directories_cb),
+	                  mf);
 
 	g_slist_foreach (mounts, (GFunc) g_free, NULL);
 	g_slist_free (mounts);
@@ -540,7 +573,7 @@ set_up_mount_point (TrackerMinerFiles *miner,
 		                        removable_device_urn, removable_device_urn);
 
 		g_string_append_printf (queries,
-		                        "INSERT INTO <%s> { ?do tracker:available true } WHERE { ?do nie:dataSource <%s> } ",
+		                        "INSERT INTO <%s> { ?do tracker:available true } WHERE { ?do nie:dataSource <%s> OPTIONAL { ?do tracker:available ?available } FILTER (!bound(?available)) } ",
 		                        removable_device_urn, removable_device_urn);
 	} else {
 		gchar *now;
@@ -564,7 +597,7 @@ set_up_mount_point (TrackerMinerFiles *miner,
 		                        removable_device_urn, removable_device_urn);
 
 		g_string_append_printf (queries,
-		                        "DELETE FROM <%s> { ?do tracker:available true } WHERE { ?do nie:dataSource <%s> } ",
+		                        "DELETE FROM <%s> { ?do tracker:available true } WHERE { ?do nie:dataSource <%s> ; tracker:available true } ",
 		                        removable_device_urn, removable_device_urn);
 		g_free (now);
 	}
@@ -649,9 +682,9 @@ query_mount_points_cb (GObject      *source,
 	g_hash_table_replace (volumes, g_strdup (TRACKER_NON_REMOVABLE_MEDIA_DATASOURCE_URN),
 	                      GINT_TO_POINTER (VOLUME_MOUNTED));
 
-	uuids = tracker_storage_get_removable_device_uuids (priv->storage);
+	uuids = tracker_storage_get_device_uuids (priv->storage, TRACKER_STORAGE_REMOVABLE, FALSE);
 
-	/* Then, get all currently mounted volumes, according to HAL */
+	/* Then, get all currently mounted volumes, according to GIO */
 	for (u = uuids; u; u = u->next) {
 		const gchar *uuid;
 		gchar *removable_device_urn;
@@ -752,22 +785,38 @@ static void
 mount_point_added_cb (TrackerStorage *storage,
                       const gchar    *uuid,
                       const gchar    *mount_point,
+                      gboolean        removable,
+                      gboolean        optical,
                       gpointer        user_data)
 {
 	TrackerMinerFiles *miner = user_data;
 	TrackerMinerFilesPrivate *priv;
+	GFile *file;
 	gchar *urn;
 	gboolean index_removable_devices;
+	gboolean index_optical_discs;
+	gboolean should_crawl;
 
 	priv = TRACKER_MINER_FILES_GET_PRIVATE (miner);
 
 	index_removable_devices = tracker_config_get_index_removable_devices (priv->config);
+	index_optical_discs = tracker_config_get_index_optical_discs (priv->config);
 
 	g_message ("Added mount point '%s'", mount_point);
 
-	if (index_removable_devices) {
-		GFile *file;
+	should_crawl = TRUE;	
 
+	if (removable && !tracker_config_get_index_removable_devices (priv->config)) {
+		g_message ("  Not crawling, removable devices disabled in config");
+		should_crawl = FALSE;
+	}
+
+	if (optical && !tracker_config_get_index_optical_discs (priv->config)) {
+		g_message ("  Not crawling, optical devices discs disabled in config");
+		should_crawl = FALSE;
+	}
+
+	if (should_crawl) {
 		g_message ("  Adding directory to crawler's queue");
 
 		file = g_file_new_for_path (mount_point);
@@ -784,8 +833,6 @@ mount_point_added_cb (TrackerStorage *storage,
 		                                file,
 		                                TRUE);
 		g_object_unref (file);
-	} else {
-		g_message ("  Not crawling, removable devices disabled in config");
 	}
 
 	urn = g_strdup_printf (TRACKER_DATASOURCE_URN_PREFIX "%s", uuid);
@@ -1101,62 +1148,45 @@ index_single_directories_cb (GObject    *gobject,
         private->index_single_directories = tracker_gslist_copy_with_string_data (new_dirs);
 }
 
+static gboolean
+miner_files_force_recheck_idle (gpointer user_data)
+{
+	TrackerMinerFiles *miner_files = user_data;
+
+	/* Recheck all directories for compliance with the new config */
+	tracker_miner_fs_force_recheck (TRACKER_MINER_FS (miner_files));
+
+	miner_files->private->force_recheck_id = 0;
+
+	return FALSE;
+}
+
+static void
+ignore_directories_cb (GObject    *gobject,
+                       GParamSpec *arg1,
+                       gpointer    user_data)
+{
+	TrackerMinerFiles *miner_files = user_data;
+
+	if (miner_files->private->force_recheck_id == 0) {
+		/* Set idle so multiple changes in the config lead to one recheck */
+		miner_files->private->force_recheck_id =
+			g_idle_add (miner_files_force_recheck_idle, miner_files);
+	}
+}
+
 static gboolean
 miner_files_check_file (TrackerMinerFS *fs,
                         GFile          *file)
 {
 	TrackerMinerFiles *mf;
-	GFileInfo *file_info;
-	GSList *l;
-	gchar *basename;
-	gchar *path;
-	gboolean should_process;
-
-	file_info = NULL;
-	should_process = FALSE;
-	basename = NULL;
-	path = NULL;
-
-	file_info = g_file_query_info (file,
-	                               G_FILE_ATTRIBUTE_STANDARD_IS_HIDDEN,
-	                               G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
-	                               NULL, NULL);
-
-	if (file_info && g_file_info_get_is_hidden (file_info)) {
-		/* Ignore hidden files */
-		goto done;
-	}
 
 	/* Check module file ignore patterns */
 	mf = TRACKER_MINER_FILES (fs);
 
-	path = g_file_get_path (file);
-
-	for (l = tracker_config_get_ignored_file_paths (mf->private->config); l; l = l->next) {
-		if (strcmp (l->data, path) == 0) {
-			goto done;
-		}
-	}
-
-	basename = g_file_get_basename (file);
-
-	for (l = tracker_config_get_ignored_file_patterns (mf->private->config); l; l = l->next) {
-		if (g_pattern_match_string (l->data, basename)) {
-			goto done;
-		}
-	}
-
-	should_process = TRUE;
-
-done:
-	g_free (basename);
-	g_free (path);
-
-	if (file_info) {
-		g_object_unref (file_info);
-	}
-
-	return should_process;
+	return tracker_miner_files_check_file (file,
+	                                       tracker_config_get_ignored_file_paths (mf->private->config),
+	                                       tracker_config_get_ignored_file_patterns (mf->private->config));
 }
 
 static gboolean
@@ -1164,105 +1194,17 @@ miner_files_check_directory (TrackerMinerFS *fs,
                              GFile          *file)
 {
 	TrackerMinerFiles *mf;
-	GFileInfo *file_info;
-	GSList *l;
-	gchar *basename;
-	gchar *path;
-	gboolean should_process;
-	gboolean is_hidden;
-
-	should_process = FALSE;
-	basename = NULL;
-
-	/* Most common things to ignore */
-	file_info = g_file_query_info (file,
-	                               G_FILE_ATTRIBUTE_STANDARD_IS_HIDDEN,
-	                               G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
-	                               NULL, NULL);
-
-	path = g_file_get_path (file);
-
-	/* First we check the GIO hidden check. This does a number of
-	 * things for us which is good (like checking ".foo" dirs).
-	 */
-	is_hidden = file_info && g_file_info_get_is_hidden (file_info);
-
-	/* Second we check if the file is on FAT and if the hidden
-	 * attribute is set. GIO does this but ONLY on a Windows OS,
-	 * not for Windows files under a Linux OS, so we have to check
-	 * anyway.
-	 */
-	if (!is_hidden) {
-		int fd;
-
-		fd = g_open (path, O_RDONLY, 0);
-		if (fd != -1) {
-			__u32 attrs;
-
-			if (ioctl (fd, FAT_IOCTL_GET_ATTRIBUTES, &attrs) == 0) {
-				is_hidden = attrs & ATTR_HIDDEN ? TRUE : FALSE;
-			}
-
-			close (fd);
-		}
-	}
-
-	if (is_hidden) {
-		TrackerMinerFiles *mf;
-		GSList *allowed_directories;
-
-		mf = TRACKER_MINER_FILES (fs);
-
-		/* FIXME: We need to check if the file is actually a
-		 * config specified location before blanket ignoring
-		 * all hidden files.
-		 */
-		allowed_directories =
-			tracker_config_get_index_recursive_directories (mf->private->config);
-
-		if (tracker_string_in_gslist (path, allowed_directories)) {
-			should_process = TRUE;
-		}
-
-		allowed_directories =
-			tracker_config_get_index_single_directories (mf->private->config);
-
-		if (tracker_string_in_gslist (path, allowed_directories)) {
-			should_process = TRUE;
-		}
-
-		/* Ignore hidden dirs */
-		goto done;
-	}
 
+	/* Check module file ignore patterns */
 	mf = TRACKER_MINER_FILES (fs);
 
-	for (l = tracker_config_get_ignored_directory_paths (mf->private->config); l; l = l->next) {
-		if (strcmp (l->data, path) == 0) {
-			goto done;
-		}
-	}
+	return tracker_miner_files_check_directory (file,
+	                                            tracker_config_get_index_recursive_directories (mf->private->config),
+	                                            tracker_config_get_index_single_directories (mf->private->config),
+	                                            tracker_config_get_ignored_directory_paths (mf->private->config),
+	                                            tracker_config_get_ignored_directory_patterns (mf->private->config));
 
-	basename = g_file_get_basename (file);
 
-	for (l = tracker_config_get_ignored_directory_patterns (mf->private->config); l; l = l->next) {
-		if (g_pattern_match_string (l->data, basename)) {
-			goto done;
-		}
-	}
-
-	/* Check module directory ignore patterns */
-	should_process = TRUE;
-
- done:
-	g_free (basename);
-	g_free (path);
-
-	if (file_info) {
-		g_object_unref (file_info);
-	}
-
-	return should_process;
 }
 
 static gboolean
@@ -1271,40 +1213,12 @@ miner_files_check_directory_contents (TrackerMinerFS *fs,
                                       GList          *children)
 {
 	TrackerMinerFiles *mf;
-	GSList *ignored_content, *l;
 
 	mf = TRACKER_MINER_FILES (fs);
-	ignored_content = tracker_config_get_ignored_directories_with_content (mf->private->config);
 
-	if (!ignored_content) {
-		return TRUE;
-	}
-
-	while (children) {
-		gchar *basename;
-
-		basename = g_file_get_basename (children->data);
-
-		for (l = ignored_content; l; l = l->next) {
-			if (g_strcmp0 (basename, l->data) == 0) {
-				gchar *parent_uri;
-
-				parent_uri = g_file_get_uri (parent);
-				g_debug ("Directory '%s' ignored since it contains a file named '%s'",
-				         parent_uri, basename);
-
-				g_free (parent_uri);
-				g_free (basename);
-
-				return FALSE;
-			}
-		}
-
-		children = children->next;
-		g_free (basename);
-	}
-
-	return TRUE;
+	return tracker_miner_files_check_directory_contents (parent,
+	                                                     children,
+	                                                     tracker_config_get_ignored_directories_with_content (mf->private->config));
 }
 
 static gboolean
@@ -1312,34 +1226,12 @@ miner_files_monitor_directory (TrackerMinerFS *fs,
                                GFile          *file)
 {
 	TrackerMinerFiles *mf;
-	GSList *list;
 
 	mf = TRACKER_MINER_FILES (fs);
 
-	if (!tracker_config_get_enable_monitors (mf->private->config)) {
-		return FALSE;
-	}
-
-	/* We don't want child directories inside IndexSingleDirectories
-	 * to have a monitor added.
-	 */
-	for (list = mf->private->index_single_directories; list; list = list->next) {
-		gboolean is_child = FALSE;
-		GFile *dir;
-
-		dir = g_file_new_for_path (list->data);
-		is_child = g_file_has_prefix (file, dir);
-		g_object_unref (dir);
-
-		if (is_child) {
-			return FALSE;
-		}
-	}
-
-	/* Fallback to the check directory routine, since we don't
-	 * monitor anything we don't process.
-	 */
-	return miner_files_check_directory (fs, file);
+	return tracker_miner_files_monitor_directory (file,
+	                                              tracker_config_get_enable_monitors (mf->private->config),
+	                                              mf->private->index_single_directories);
 }
 
 static const gchar *
@@ -1778,3 +1670,225 @@ tracker_miner_files_new (TrackerConfig *config)
 	                     "process-pool-limit", 10,
 	                     NULL);
 }
+
+gboolean
+tracker_miner_files_check_file (GFile  *file,
+                                GSList *ignored_file_paths,
+                                GSList *ignored_file_patterns)
+{
+	GFileInfo *file_info;
+	GSList *l;
+	gchar *basename;
+	gchar *path;
+	gboolean should_process;
+
+	file_info = NULL;
+	should_process = FALSE;
+	basename = NULL;
+	path = NULL;
+
+	file_info = g_file_query_info (file,
+	                               G_FILE_ATTRIBUTE_STANDARD_IS_HIDDEN,
+	                               G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+	                               NULL, NULL);
+
+	if (file_info && g_file_info_get_is_hidden (file_info)) {
+		/* Ignore hidden files */
+		goto done;
+	}
+
+
+	path = g_file_get_path (file);
+
+	for (l = ignored_file_paths; l; l = l->next) {
+		if (strcmp (l->data, path) == 0) {
+			goto done;
+		}
+	}
+
+	basename = g_file_get_basename (file);
+
+	for (l = ignored_file_patterns; l; l = l->next) {
+		if (g_pattern_match_string (l->data, basename)) {
+			goto done;
+		}
+	}
+
+	should_process = TRUE;
+
+done:
+	g_free (basename);
+	g_free (path);
+
+	if (file_info) {
+		g_object_unref (file_info);
+	}
+
+	return should_process;
+}
+
+gboolean
+tracker_miner_files_check_directory (GFile  *file,
+                                     GSList *index_recursive_directories,
+                                     GSList *index_single_directories,
+                                     GSList *ignored_directory_paths,
+                                     GSList *ignored_directory_patterns)
+{
+	GFileInfo *file_info;
+	GSList *l;
+	gchar *basename;
+	gchar *path;
+	gboolean should_process;
+	gboolean is_hidden;
+
+	should_process = FALSE;
+	basename = NULL;
+
+	/* Most common things to ignore */
+	file_info = g_file_query_info (file,
+	                               G_FILE_ATTRIBUTE_STANDARD_IS_HIDDEN,
+	                               G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS,
+	                               NULL, NULL);
+
+	path = g_file_get_path (file);
+
+	/* First we check the GIO hidden check. This does a number of
+	 * things for us which is good (like checking ".foo" dirs).
+	 */
+	is_hidden = file_info && g_file_info_get_is_hidden (file_info);
+
+	/* Second we check if the file is on FAT and if the hidden
+	 * attribute is set. GIO does this but ONLY on a Windows OS,
+	 * not for Windows files under a Linux OS, so we have to check
+	 * anyway.
+	 */
+	if (!is_hidden) {
+		int fd;
+
+		fd = g_open (path, O_RDONLY, 0);
+		if (fd != -1) {
+			__u32 attrs;
+
+			if (ioctl (fd, FAT_IOCTL_GET_ATTRIBUTES, &attrs) == 0) {
+				is_hidden = attrs & ATTR_HIDDEN ? TRUE : FALSE;
+			}
+
+			close (fd);
+		}
+	}
+
+	if (is_hidden) {
+		/* FIXME: We need to check if the file is actually a
+		 * config specified location before blanket ignoring
+		 * all hidden files.
+		 */
+		if (tracker_string_in_gslist (path, index_recursive_directories)) {
+			should_process = TRUE;
+		}
+
+		if (tracker_string_in_gslist (path, index_single_directories)) {
+			should_process = TRUE;
+		}
+
+		/* Ignore hidden dirs */
+		goto done;
+	}
+
+	for (l = ignored_directory_paths; l; l = l->next) {
+		if (strcmp (l->data, path) == 0) {
+			goto done;
+		}
+	}
+
+	basename = g_file_get_basename (file);
+
+	for (l = ignored_directory_patterns; l; l = l->next) {
+		if (g_pattern_match_string (l->data, basename)) {
+			goto done;
+		}
+	}
+
+	/* Check module directory ignore patterns */
+	should_process = TRUE;
+
+ done:
+	g_free (basename);
+	g_free (path);
+
+	if (file_info) {
+		g_object_unref (file_info);
+	}
+
+	return should_process;
+}
+
+gboolean
+tracker_miner_files_check_directory_contents (GFile  *parent,
+                                              GList  *children,
+                                              GSList *ignored_content)
+{
+	GSList *l;
+
+	if (!ignored_content) {
+		return TRUE;
+	}
+
+	while (children) {
+		gchar *basename;
+
+		basename = g_file_get_basename (children->data);
+
+		for (l = ignored_content; l; l = l->next) {
+			if (g_strcmp0 (basename, l->data) == 0) {
+				gchar *parent_uri;
+
+				parent_uri = g_file_get_uri (parent);
+				/* g_debug ("Directory '%s' ignored since it contains a file named '%s'", */
+				/*          parent_uri, basename); */
+
+				g_free (parent_uri);
+				g_free (basename);
+
+				return FALSE;
+			}
+		}
+
+		children = children->next;
+		g_free (basename);
+	}
+
+	return TRUE;
+}
+
+gboolean
+tracker_miner_files_monitor_directory (GFile    *file,
+                                       gboolean  enable_monitors,
+                                       GSList   *directories_to_check)
+{
+	GSList *l;
+
+	if (!enable_monitors) {
+		return FALSE;
+	}
+
+	/* We don't want child directories inside IndexSingleDirectories
+	 * to have a monitor added.
+	 */
+	for (l = directories_to_check; l; l = l->next) {
+		gboolean is_child = FALSE;
+		GFile *dir;
+
+		dir = g_file_new_for_path (l->data);
+		is_child = g_file_has_prefix (file, dir);
+		g_object_unref (dir);
+
+		if (is_child) {
+			return FALSE;
+		}
+	}
+
+	/* Fallback to the check directory routine, since we don't
+	 * monitor anything we don't process.
+	 */
+	return TRUE;
+}
